package NTLSystem.support;

import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A nice "About" Dialog box.  Has set-aside room for the program name
 * and version number, as well as a text box for any extra text.
 * <P>
 * $Id: AboutDialog.java,v 1.2 1998/07/21 01:27:21 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author Josh Sled [jsled@icsi.berkeley.edu]
 * @version $Revision: 1.2 $
 **/
public class AboutDialog
extends JDialog
implements ActionListener {
  /** 
   * The constructor for a standard, simple "About" dialog.
   *
   * @param     f The parent frame.
   * @param     progName The name of the program this dialog is
   *            describing.
   * @param     version The version number of the program, or null if
   *            one does not exist.
   * @param     text The text which describes the program.
   **/
  public AboutDialog(Frame f,
		     String progName,
		     String version,
		     String text)
  {
    super(f, "About " + progName, false);
    
    JPanel p;
    JButton b;
    GridBagLayout gbl = new GridBagLayout();
    GridBagLayout gbl2 = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.gridwidth=GridBagConstraints.REMAINDER;
    c.gridheight=1;
    c.anchor = GridBagConstraints.CENTER;

    getContentPane().setLayout(gbl);

    // Set the program and version number [if existant] fields.
    JLabel l;
    p = new JPanel();
    p.setLayout(gbl2);
    l = new JLabel(progName, JLabel.CENTER);
    gbl2.setConstraints(l, c);
    p.add(l);

    if (version != null) {
      l = new JLabel("Version " + version, JLabel.CENTER);
      gbl2.setConstraints(l, c);
      p.add(l);
    }
    gbl.setConstraints(p, c);
    getContentPane().add(p);
    
    p = new JPanel();
    JTextArea ta = new JTextArea(text, 10, 60);
    ta.setEditable(false);
    p.add(ta);
    gbl.setConstraints(p, c);
    getContentPane().add(p);
    
    p = new JPanel();
    b = new JButton("Dismiss");
    b.addActionListener(this);
    p.add(b);
    gbl.setConstraints(p, c);
    getContentPane().add(p);

    pack();
    setVisible(true);
  } // end AboutDialog(...)

  public void actionPerformed(ActionEvent ae)
  {
    // We got the "Ok" button press...
    // Destory the about dialog.
    setVisible(false);
    dispose();
  }

} // end public class AboutDialog



