package NTLSystem.support;

import java.beans.*;
import java.util.*;

/**
 * A basic MultivaluedAttribute.  This is used to test the
 * PropertyCollectionSheet and the editing framework.  It's probably
 * useless otherwise.
 * <P>
 * $Id: MultivaluedAttribute.java,v 1.2 1998/06/12 19:25:11 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author Josh Sled [jsled@icsi.berkeley.edu]
 * @see MultivaluedAttributeEditor
 * @see PropCollSheetTest
 * @version $Revision: 1.2 $
 **/
public class MultivaluedAttribute 
{
  /** The possible values in this attribute. **/
  private String possibleValues[];
  /** A sentinel value indicating no value for this attribute. **/
  private static final int NOVALUE = -1;
  /** The index of the current value. **/
  private int curValue = NOVALUE;

  /** Constructor **/
  public MultivaluedAttribute(String values[])
  {
    possibleValues = values;
  }

  /**
   * Returns the number of possible value in this multivalued
   * attribute.
   *
   * @return    The number of possible values for this attribute.
   **/ 
  public int getValueCount()
  {
    return possibleValues.length;
  }


  /**
   * Returns an enumeration of the number of possible values.
   *
   * @return    An enumermation of the possible values of this
   *            attribute. 
   **/
  public Enumeration getPossibleValues()
  {
    return new Enumeration() {
      int i=0;
      
      public boolean hasMoreElements() {
	return (i < possibleValues.length);
      } 

      public Object nextElement() {
	Object toRet = possibleValues[i];
	i+=1;
	return toRet;
      }
    }; // end anonymous Enumeration class
  }

  /**
   * Returns the current value of this attribute.
   *
   * @return    The current value.
   **/
  public String getValue() 
  {
    if ( (curValue == NOVALUE) || (curValue > possibleValues.length-1)) {
      return null;
    }
    return new String(possibleValues[curValue]);
  }

  /**
   * Sets the value of this attribute to the given String.
   *
   * @param     s The string to set the attribute to.
   * @exception NoSuchElementException If the given String is not a
   *            valid value for the attribute.
   **/
  public void setValue(String s)
       throws NoSuchElementException
  {
    for (int i=0; i < possibleValues.length ; i++)
      {
	if (possibleValues[i].equals(s)) {
	  curValue = i;
	  return;
	}
      }
    throw new NoSuchElementException("No element " + s + " in MultivaluedAttribute");
  }

  public String toString()
  {
    String toRet = null;

    if ( (curValue == NOVALUE) || (curValue > possibleValues.length) ) {
      toRet = new String("");
    } else {
      toRet = new String(possibleValues[curValue]);
    } // end if-else
    return toRet;
  }


  /** 
   * Self-test code.  Enjoy.
   **/
  public static void main(String args[])
  {

    String names[] = {
      new String("Foo"),
      new String("Bar"),
      new String("Baz")
    };

    MultivaluedAttribute ma = new MultivaluedAttribute(names);
    ma.setValue("Foo");

    System.out.println("Number of items in MultivaluedAttribute: " + ma.getValueCount());
    for (Enumeration e = ma.getPossibleValues(); e.hasMoreElements(); ) {
      System.out.println(e.nextElement());
    } // end for

  } 
	
}

