package NTLSystem.support;

import java.awt.*;
import java.awt.event.*;

/**
 * Utility class to catch window close events on a target
 * window and actually dispose the window.
 * <P>
 * Stolen from the BDK, but very useful.
 * <P>
 * $Id: WindowCloser.java,v 1.2 1998/06/17 23:01:03 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @version $Revision: 1.2 $
 **/
public class WindowCloser implements WindowListener {

   /**
    * Create an adaptor to listen for window closing events
    * on the given window and actually perform the close.
    *
    * @param     w The Window to associate this WindowListener with.
    **/
    public WindowCloser(Window w) {
	this(w, false);
    }

   /**
    * Create an adaptor to listen for window closing events
    * on the given window and actually perform the close.
    *
    * @param     w The Window to associate this WindowListener with.
    * @param     exitOnClose If true we do a System.exit on close.
    */
    public WindowCloser(Window w, boolean exitOnClose) {
	this.exitOnClose = exitOnClose;
	w.addWindowListener(this);
    }


    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
	if (exitOnClose) {
            System.exit(0);
	}
	e.getWindow().dispose();
    }

    public void windowClosed(WindowEvent e) {
	if (exitOnClose) {
            System.exit(0);
	}
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
    }

    public void windowActivated(WindowEvent e) {
    }

    public void windowDeactivated(WindowEvent e) {
    }

    private boolean exitOnClose;
}
