package NTLSystem.verblearn;

import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import NTLSystem.support.*;

/**
 * A tuple of label, sense, and prior, likelihood and posterior
 * probabilities.  (This allows Slot.bestLabelFor() to essentially
 * return multiple values.)
 * <P>
 * <STRONG>NOTE: Some users of this class do not set the prior or
 * likelihood fields.</STRONG>
 * <P>
 * <EM>This class needn't be public... it is only to get Javadoc 1.1
 * to generate docs for it.</EM>
 * <P>
 * $Id: CandidateSense.java,v 1.1 1998/07/31 21:11:26 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    David Bailey [dbailey@icsi.berkeley.edu]
 * @author    Joshua Sled [jsled@icsi.berkeley.edu]
 * @see       Sense
 * @see       VerbComplex
 * @version   $Revision: 1.1 $
 **/
public class CandidateSense
    implements Serializable
{

    public VerbComplex label;
    public Sense       sense;
    public double      prior;
    public double      likelihood;
    public double      posterior;

    /**
     * Basic no-arg constructor for serialization.
     **/
    CandidateSense() {}

    /**
     * This constructor leaves the prior and likelihood slots
     * unused. 
     *
     * @param     label The Label for this CandidateSense.
     * @param     sense The Sense which is a Candidate.
     * @param     posterior The posterior for this CandidateSense.
     **/
    CandidateSense(VerbComplex label,
		   Sense sense,
		   double posterior) {
	this.label = label;
	this.sense = sense;
	this.posterior = posterior;
    }

    /**
     * A full constructor for CandidateSenses.
     *
     * @param     label The Label for this CandidateSense.
     * @param     sense The Sense which is a Candidate.
     * @param     prior The prior for this CandidateSense.
     * @param     likelihood The likelihood of this CandidateSense.
     * @param     posterior The posterior for this CandidateSense.
     **/
    CandidateSense(String label, Sense sense, double prior,
		   double likelihood, double posterior)
    {
	this(label, sense, posterior);
	this.prior = prior;
	this.likelihood = likelihood;
    }

    public String toString() {
	return "sense = " + sense.id() + 
	    ", prior = " + Fmt.dbl(prior) +
	    ", likelihood = " + Fmt.dbl(likelihood) +
	    ", posterior = " + Fmt.dbl(posterior) + "\n";
    }
}
