package NTLSystem.verblearn;

import com.sun.java.swing.*;
import com.sun.java.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.Enumeration;
import java.util.Vector;
import NTLSystem.support.*;

public class TrainingGUI
    extends JInternalFrame
{
    private JComboBox langCbox;
    private Vector    paramsVec;
    private JComboBox parmsCbox;
    private JComboBox lexiconCbox;
    private Vector    dsetsVec;
    private JComboBox dataSetCbox;
    private JButton   trainButton;
    private JButton   logSaveButton;
    private JButton   logClearButton;
    private JTextArea trainingOutput;
    private PropertyCollectionSheetPanel params;

    protected ScenarioSet sset;
    protected VerbLearn vl;

    public TrainingGUI(ScenarioSet ss, VerbLearn vl)
    {
	super("Training", true, true, true, true);

    sset = ss;
    this.vl = vl;

    // Before we do anything else, set up the correct editors for the
    // Params editing, or it won't work.
    // Set the Integer editor to "IntEditor", the Boolean editor to
    // "BoolEditor" and the Double editor to "DoubleEditor".
    try {
      PropertyEditorManager.
	  registerEditor(Class.forName("java.lang.Integer"),
			 Class.forName("sun.beans.editors.IntEditor"));
      PropertyEditorManager.
	  registerEditor(Class.forName("java.lang.Boolean"),
			 Class.forName("sun.beans.editors.BoolEditor"));
      PropertyEditorManager.
	  registerEditor(Class.forName("java.lang.Double"),
			 Class.forName("NTLSystem.support.DoubleEditor"));
    } catch (ClassNotFoundException cnfe) {
      System.err.println("Could not register editors.");
      System.err.println(cnfe.getMessage());
      cnfe.printStackTrace();
    } // end try-catch

    GridBagLayout gbl, gblTmp;
    GridBagConstraints c = new GridBagConstraints();
    JPanel jp;
    JLabel jl;
    JButton jb;
    JComponent jc = (JComponent)getContentPane();

    gbl = new GridBagLayout();
    jc.setLayout(gbl);


    // Top-left panel: Language selection, Lexicon selection, Dataset
    // selection. 
    gblTmp = new GridBagLayout();
    jp = new JPanel(gblTmp);

    c.gridwidth = 1;
    jl = new JLabel("Language:");
    gblTmp.setConstraints(jl, c);
    jp.add(jl);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.HORIZONTAL;
    langCbox =
	new JComboBox( new VectorComboBoxModel( sset.langs ) );
    gblTmp.setConstraints(langCbox, c);
    jp.add(langCbox);

    c.gridwidth = 1;
    c.fill = GridBagConstraints.NONE;
    jl = new JLabel("Lexicon:");
    gblTmp.setConstraints(jl, c);
    jp.add(jl);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.HORIZONTAL;
    // Extend the basic VectorComboBoxModel to correctly handle the
    // lexicon names.  
    lexiconCbox =
	new JComboBox( new VectorComboBoxModel(sset.lexicons) {
	    public Object getElementAt( int i )
	    { return ((Lexicon)super.getElementAt(i)).getName(); }
	} );
    // Set this editable to allow the user to input a new Lexicon
    // name.
    lexiconCbox.setEditable(true);
    gblTmp.setConstraints(lexiconCbox, c);
    jp.add(lexiconCbox);

    c.gridwidth = 1;
    c.fill = GridBagConstraints.NONE;
    jl = new JLabel("Data set:");
    gblTmp.setConstraints(jl, c);
    jp.add(jl);

    c.gridwidth = GridBagConstraints.RELATIVE;
    // Create a local copy of the dataset names.
    dsetsVec = new Vector();
    Enumeration denum = sset.datasets.getSortedKeys();
    while ( denum.hasMoreElements() ) {
	dsetsVec.addElement( denum.nextElement() );
    } // end while over datasets enumeration
    dataSetCbox = new JComboBox( dsetsVec );
    Dataset dsetSel = sset.getDataset();
    if ( dsetSel != null ) {
	dataSetCbox.setSelectedItem( dsetSel.getName() );
    }
    gblTmp.setConstraints(dataSetCbox, c);
    jp.add(dataSetCbox);

    c.gridwidth = GridBagConstraints.REMAINDER;
    jb = new JButton("Edit");
    jb.addActionListener( new ActionListener() {
	public void actionPerformed(ActionEvent ae) {
	    DatasetEditor de =
		new DatasetEditor( sset.getDataset((String)dataSetCbox.getSelectedItem()),
				   sset );
	    TrainingGUI.this.vl.desktop.add( de,
					     JLayeredPane.DEFAULT_LAYER );
	    TrainingGUI.this.vl.desktop.moveToFront( de );
	}
    } );
    gblTmp.setConstraints(jb, c);
    jp.add(jb);

    // Add 'train' button
    c.fill = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    trainButton = new JButton("Train");
    trainButton.addActionListener( new ActionListener() {
	public void actionPerformed(ActionEvent ae) {
	    // Do training on seperate thread... 
	    Thread t = new Thread() {
		public void run() { 
		    trainButton.setEnabled( false );
		    train();
		    JOptionPane.showMessageDialog( TrainingGUI.this,
						   "Done Training.",
						   "Done Training.",
						   JOptionPane.INFORMATION_MESSAGE );
		    trainButton.setEnabled( true );
		}
	    }; // end Thread
	    t.start(); 
	}
    } );
    gblTmp.setConstraints(trainButton, c);
    jp.add( trainButton );

    JPanel jp2 = new JPanel();
    logSaveButton = new JButton( "Save Log" );
    logSaveButton.addActionListener( new ActionListener() {
	public void actionPerformed(ActionEvent ae) {
	    // Save the log to a specified file.
	    // Get the file name from the user.
	    JFileChooser jfc = new JFileChooser();
	    jfc.setDialogTitle( "Save training log as..." );
	    int status = jfc.showSaveDialog( TrainingGUI.this.vl );
	    if ( status != JFileChooser.CANCEL_OPTION ) {
		File toSaveTo = jfc.getSelectedFile();
		boolean cont = true;
		if ( toSaveTo.exists() ) {
		    cont = (JOptionPane.showConfirmDialog( TrainingGUI.this.vl,
							   "Overwrite file " +
							   toSaveTo.getPath() +
							   "?",
							   "Overwrite file",
							   JOptionPane.YES_NO_OPTION)
			    == JOptionPane.YES_OPTION);
		} // end if
		if (cont) {
		    try {
			FileWriter fw =
			    new FileWriter( toSaveTo );
			Utility.SaveTextArea( trainingOutput, fw );
			fw.close();
			JOptionPane.showMessageDialog( TrainingGUI.this.vl,
						       "Done saving log.",
						       "Done saving log.",
						       JOptionPane.INFORMATION_MESSAGE );
		    } catch (IOException ioe) {
			JOptionPane.showMessageDialog( TrainingGUI.this.vl,
						       "Couldn't save file " +
						       toSaveTo.getPath() +
						       " because " +
						       ioe.getMessage(),
						       "File save error",
						       JOptionPane.ERROR_MESSAGE );
		    } // end-catch
		} // end cont
	    } // end-if
	    // Output the contents of the log to the file.
	}
    } );
    jp2.add( logSaveButton );

    logClearButton = new JButton( "Clear Log" );
    logClearButton.addActionListener( new ActionListener() {
	public void actionPerformed(ActionEvent ae) {
	    // Clear the log...
	    Document d = trainingOutput.getDocument();
	    try {
		d.remove( 0, d.getLength() );
	    } catch (BadLocationException ble) {
		// We don't expect this because of the brain-dead
		// arguments we're using, but it might happen.
		throw new InternalError( ble.getMessage() );
	    } // end try-catch
	    trainingOutput.validate();
	    //	    trainingOutput.update( trainingOutput.getGraphics() );
	}
    } );
    jp2.add( logClearButton );

    c.fill = GridBagConstraints.HORIZONTAL;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.anchor = GridBagConstraints.CENTER;
    gblTmp.setConstraints( jp2, c );
    jp.add( jp2 );
    
    // Add the top-left panel.
    c.gridwidth = GridBagConstraints.RELATIVE;
    c.weightx = 0.3;
    c.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(jp, c);
    jc.add(jp);


    // Top-right panel: Parameter set selection and parameter editing.

    gblTmp = new GridBagLayout();
    jp = new JPanel(gblTmp);

    jp.setBorder(BorderFactory.createTitledBorder("Parameters"));

    c.gridwidth = GridBagConstraints.RELATIVE;
    jl = new JLabel("Name:");
    gblTmp.setConstraints(jl, c);
    jp.add(jl);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.HORIZONTAL;
    paramsVec = new Vector();
    Enumeration e = sset.params.getSortedKeys();
    while ( e.hasMoreElements() ) {
	paramsVec.addElement( e.nextElement() );
    } // end while over params Enumeration
    parmsCbox = new JComboBox( paramsVec );
    Params selParams = sset.getParams();
    if ( selParams != null ) {
	parmsCbox.setSelectedItem( selParams.getName() );
    }
    
    // Set this ComboBox to be editable so that the name of the parameters
    // can be changed.
    parmsCbox.setEditable(true);
    // Register an actionListener so selections [especially
    // user-typed-in selections] cause a new Params object to be
    // created and made able to be edited.
    parmsCbox.getEditor().
	addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		String paramsName =
		    (String)parmsCbox.getSelectedItem();
		System.out.println("Creating params named " +
				   paramsName );
		// If new, create a new Params object with that
		// name and add it to the ScenarioSet 
		if ( sset.getParams( paramsName ) == null ) {
		    sset.addParams( new Params( paramsName, sset ) );
		    parmsCbox.addItem( paramsName );
		}
		parmsCbox.validate();
		parmsCbox.update( parmsCbox.getGraphics() );
		params.setTarget( sset.getParams(paramsName) );
		params.validate();
	    }
    } );
    parmsCbox.addItemListener( new ItemListener() {
	public void itemStateChanged(ItemEvent ie) {
	    String itemName = (String)parmsCbox.getSelectedItem();
	    params.setTarget( sset.getParams(itemName) );
	    parmsCbox.validate();
	    parmsCbox.update( parmsCbox.getGraphics() );
	}
    } );
    gblTmp.setConstraints(parmsCbox, c);
    jp.add(parmsCbox);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.BOTH;
    c.weightx = c.weighty = 1.0;
    Params p = sset.getParams();
    params = new PropertyCollectionSheetPanel( p );
    params.setSaveAsEdited( true );
    JScrollPane jsp = new JScrollPane( params );
    jsp.setPreferredSize( new Dimension(350, 200) );
    gblTmp.setConstraints(jsp, c);
    jp.add(jsp);

    // Add the top-right panel.
    c.fill = GridBagConstraints.BOTH;
    c.weightx = 0.7;
    c.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(jp, c);
    jc.add(jp);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.weightx = c.weighty = 1.0;
    c.fill = GridBagConstraints.BOTH;
    trainingOutput = new AppendScrollingTextArea(20, 50);
    trainingOutput.setEditable( false );
    jsp  = new JScrollPane( trainingOutput,
			    JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			    JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    jsp.setBorder( BorderFactory.
		   createTitledBorder("Training log") );
    gbl.setConstraints(jsp, c);
    jc.add( jsp );

    // Pad the whole content pane a bit.
    jc.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

    pack();
    setVisible(true);
  }


    protected void train()
    {
	// Check that _some_ lexicon is selected
	String selLexicon = (String)lexiconCbox.getSelectedItem();
	if ( (selLexicon == null) ) {
	    JOptionPane.showMessageDialog(this,
					  "You must select or enter the " +
					  "name of a Lexicon to use/create.",
					  "No Lexicon selected.",
					  JOptionPane.ERROR_MESSAGE );
	    return;
	}
	String lang = (String)langCbox.getSelectedItem();
	if ( (lang == null) ||  lang.equals("") ) {
	    JOptionPane.showMessageDialog( this,
					   "You must select a Language " +
					   "for this training run.",
					   "No Language selected.",
					   JOptionPane.ERROR_MESSAGE );
	    return;
	}
	String params = (String)parmsCbox.getSelectedItem();
	if ( params == null ) {
	    JOptionPane.showMessageDialog( this,
					   "You must select Parameters " +
					   "for this training run.",
					   "No Parameters selected.",
					   JOptionPane.ERROR_MESSAGE );
	    return;
	}
	Params p = sset.getParams( params );
	if ( p == null ) {
	    JOptionPane.showMessageDialog( this,
					   "You must select a valid " +
					   "Parameters object for this " +
					   "training run.",
					   "No valid Parameters selected.",
					   JOptionPane.ERROR_MESSAGE );
	    return;
	}

	Lexicon l = sset.getLexicon( selLexicon );
	if (l == null) {
	    l = new Lexicon( selLexicon,
			     (String)langCbox.getSelectedItem(),
			     sset,
			     p );
	}

	trainingOutput.append("Beginning training...\n\n");
	for (int i = 0; i < sset.getParams().trainingPasses(); i++) {
	    Enumeration scEnum =
		sset.getDataset( (String)dataSetCbox.getSelectedItem() ).
		trainEnum();
	    while (scEnum.hasMoreElements()) {
		Scenario sc = (Scenario)scEnum.nextElement();
			
		if ( sc.isLabeled( lang ) ) {
		    trainingOutput.append("Incorporating scenario " +
					  sc.getName() + ":\n");
		    l.incorporate( sc.getUnifiedFstruct(), 
				   sc.getLabel(lang),
				   trainingOutput );
		    // SOME CODE TO HELP DIAGNOSE LINUX MEMORY LEAK PROBLEMS:
		    // System.gc();
		    // long mTotal = Runtime.getRuntime().totalMemory();
		    // long mFree = Runtime.getRuntime().freeMemory();
		    // System.err.println("total=" + mTotal + ", free=" +
		    //		        mFree + ", thus used="
		    //		        + (mTotal - mFree));
		} else {
		    trainingOutput.append("WARNING: scenario "
					  + sc.getName() +
					  " is unlabelled.\n"); 
		}
	    } // end while [ over scEnum ]
	} // end for [ over Params.trainingPasses() ]
	// Now, synch up the Lexicon to actually do the model merging,
	// etc.
	l.synch( trainingOutput );
	trainingOutput.append("\n\nDone training.\n");
	
	// Now that it's made, add the lexicon to the ScenarioSet.
	sset.addLexicon( l );

	// Tell the LexiconPanel and Recognize/Obey menu items that
	// the lexicon has changed: 
	vl.recognize.setEnabled( true );
	vl.obey.setEnabled( true );
	vl.buildLexiconMenu();
    } // end train()
}
