package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;

/**
 * Holds a verb complex, i.e. a label for an action.  It's basically a
 * String[] extended to have a decent toString() routine.  Note: when
 * converting to/from a String, we use dashes to separate slots, even
 * empty ones.  E.g., "pick-up" and "push-".
 * <P>
 * <EM>This class needn't be public... it is only to get Javadoc 1.1
 * to generate docs for it.</EM>
 * <P>
 * $Id: VerbComplex.java,v 1.4 1998/07/31 21:09:02 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    David Bailey [dbailey@icsi.berkeley.edu]
 * @author    Nancy Chang [nchang@icsi.berkeley.edu]
 * @see       String
 * @version   $Revision: 1.4 $
 **/
public class VerbComplex
    implements Serializable
{

    /**
     * The data this VerbComplex holds: an array of Strings [slots]
     * which are filled with data.
     **/
    private String[] words;

    /**
     * Create an empty VerbComplex given a size.
     *
     * @param     size The number of slots in the VerbComplex.
     **/
    public VerbComplex(int size) { words = new String[size]; }
    
    /**
     * Create a VerbComplex from a String.
     *
     * @param     str The String to deconstruct into a VerbComplex.
     **/
    public VerbComplex(String str) {
	// This is useful for reading from text, but also for reading
	// from the labeling GUI.  -- jsled
	Vector v = new Vector();
       	String prevToken = "-";
	StringTokenizer st = new StringTokenizer(str, "-", true);
	while (st.hasMoreTokens()) {
	    String token = st.nextToken();
	    if (!token.equals("-")) {
		v.addElement(token);
	    } else { // slot is empty
		if (prevToken.equals("-")) {
		    v.addElement(null);
		}
	    }
	    prevToken = token;
	}
	words = new String[v.size()];
	for (int i = 0; i < v.size(); i++) {
	    words[i] = (String)v.elementAt(i);
	}
    }

    /**
     * Return the size of the VerbComplex [number of slots].
     *
     * @return    The number of slots in this VerbComplex.
     **/
    public int size() { return words.length; }

    /**
     * Sets a specific slot in the VerbComplex.
     *
     * @param     i The index of the slot to set.
     * @param     word The word to set it to.
     **/
    public void setWord(int i, String word) { words[i] = word; }

    /**
     * Returns the value of a specific slot in the VerbComplex.
     *
     * @param     i The index of the slot of interest.
     * @return    The String corresponding to that slot [or null if
     *            that slot is not filled].
     **/
    public String getWord(int i) { return words[i]; }

    /**
     * Queries the existance of a String for a given slot.
     *
     * @param     i The index of the slot of interest.
     * @return    True if a String is in that slot, False otherwise.
     **/
    public boolean slotFilled(int i) { return (words[i] != null); }

    // {{{ toString
    /**
     * Redefinition of Object.toString()
     **/
    public String toString() {
	String str = "";
	for (int i = 0; i < words.length; i++) {
	    if (i > 0) str += "-";
	    if (words[i] != null) str += words[i];
	}
	return str;
    }
    // }}} toString()

    // {{{ equals()
    /**
     * Redefinition of Object.equals(Object)
     **/
    public boolean equals(Object o) {
	if (! super.equals(o)) { return false; }
	VerbComplex vc2 = (VerbComplex)o;
	for (int i = 0; i < words.length; i++) {
	    if (slotFilled(i)) {
		if (!words[i].equals(vc2.words[i])) {
		    return false;
		}
	    }
	}
	return true;
    }
    // }}} equals()
}
