package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;

    
///////////////////////////////////////////////////////////////////////
// A panel containing a grid of buttons, one for each word in the
// lexicon, arranged as one column per slot.

class LexiconGrid extends Panel {

  public static final int MAX_PER_COLUMN = 10;

  private Label warn;
  private Vector buttonVec;

  LexiconGrid() {
    setLayout(new GridBagLayout());
    rebuild();
  }

  public void rebuild() {
    // Get rid of any old components:
    if (warn != null) {
      remove(warn);
      warn = null;
    }
    else if (buttonVec != null) {
      Enumeration bEnum = buttonVec.elements();
      while (bEnum.hasMoreElements()) {
	remove((Button)bEnum.nextElement());
      }
      buttonVec = null;
    }
    // Insert a new set of components:
    if (VerbLearn.context().lexicon() == null) {
      warn = new Label("(No lexicon exists yet)");
      Constrain.add(this, warn, 0, 0, 1, 1, 10, 10, 10, 10);
    }
    else {
      buttonVec = new Vector();
      int curCol = 0;
      for (int s = 0; s < VerbLearn.context().numSlots(); s++) {
	String[] words = VerbLearn.context().lexicon().slot(s).wordList();
	int curRow = 0;
	for (int w = 0; w < words.length; w++) {
	  Button b = new Button(words[w]);
	  b.setFont(new Font("Helvetica", Font.PLAIN, 10));
	  Constrain.add(this, b, curCol, curRow, 1, 1, 3, 5, 3, 5);
	  buttonVec.addElement(b);
	  curRow++;
	  if (curRow == MAX_PER_COLUMN) {
	    curCol++;
	    curRow = 0;
	  }
	}
	curCol++;
      }
    }
    // validate();
  }
    
  // Returns -1 if o is not a member button.
  public int indexOfButton(Object o) { return buttonVec.indexOf(o); }
}
