package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;


///////////////////////////////////////////////////////////////////////
// Lexicon Panel

class LexiconPanel extends Panel {

  private static final String INITIAL_TEXT = "Word display window\n\n";
  private Label title, slotTitle;
  private LangLabel langLabel;
  private ScrollingTextArea slotInfo;
  private LexiconGrid lexiconGrid;
  private TextArea text;
  private Label wordTitle;
  private Button formatSwitch;
  private Button saveLog;
  private Button saveLex;
  private String currentWord;

  LexiconPanel() {
    setLayout(new GridBagLayout());
    title = new Label("Lexicon Inspection Window");
    title.setFont(new Font("Helvetica", Font.BOLD, 18));
    Constrain.add(this, title, 0, 0, 4, 1, 10, 10, 10, 10);
    langLabel = new LangLabel();
    Constrain.add(this, langLabel, 0, 1, 4, 1, 10, 10, 10, 10);
    slotTitle = new Label("Slot Info:");
    Constrain.add(this, slotTitle, 0, 2, 4, 1, 10, 10, 0, 10);
    slotInfo = new ScrollingTextArea(null, 5, 90);
    Constrain.add(this, slotInfo, 0, 3, 4, 1, GridBagConstraints.HORIZONTAL,
		  GridBagConstraints.CENTER, 10, 10, 10, 10);
    wordTitle = new Label("Word Info:");
    Constrain.add(this, wordTitle, 0, 4, 4, 1, 10, 10, 0, 10);
    lexiconGrid = new LexiconGrid();
    Constrain.add(this, lexiconGrid, 0, 5, 1, 1, GridBagConstraints.BOTH,
		  GridBagConstraints.CENTER, 10, 10, 10, 10);
    text = new TextArea(INITIAL_TEXT, 27, 70);
    Constrain.add(this, text, 1, 5, 3, 1, 10, 10, 10, 10);
    formatSwitch = new Button("Toggle Verbosity");
    Constrain.add(this, formatSwitch, 1, 6, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.WEST, 10, 10, 10, 10);
    saveLog = new Button("Save Text Version of Lexicon...");
    Constrain.add(this, saveLog, 2, 6, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.EAST, 10, 40, 10, 10);
    saveLex = new Button("Save Lexicon...");
    Constrain.add(this, saveLex, 3, 6, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.EAST, 10, 10, 10, 10);
  }

  // Called whenever lexicon changes (e.g., by TrainPanel).
  // Also updates the slot info window.
  public void rebuildLexiconGrid() { 
    lexiconGrid.rebuild();
    text.setText(INITIAL_TEXT);
    currentWord = null;
    if (VerbLearn.context().lexicon() != null) {
      slotInfo.setText(VerbLearn.context().lexicon().toString());
    }
    else {
      slotInfo.setText("No lexicon exists yet.");
    }
  }

  public boolean action(Event e, Object junk) {
    if (e.target == formatSwitch) {
      switchFormats();
      return true;
    }
    if (e.target == saveLex) {
      if (VerbLearn.isApplication()) {
	saveLexicon();
      }
      else {
	text.setText("Sorry, applet version can't save files.\n");
      }
      return true;
    }
    if (e.target == saveLog) {
      if (VerbLearn.isApplication()) {
	saveLexiconLog();
      }
      else {
	text.setText("Sorry, applet version can't save files.\n");
      }
      return true;
    }
    int i = lexiconGrid.indexOfButton(e.target);
    if (i >= 0) {
      showWord(((Button)e.target).getLabel());
      return true;
    }
    return super.action(e, junk);
  }

  private void switchFormats() {
    Params p = VerbLearn.context().params();
    if (p != null) {
      p.setVerboseSenses(!p.verboseSenses());
      if (currentWord != null) {
	showWord(currentWord);
      }
    }
  }


  // NC note: this is the function to change if some version of browser can be used instead.
  private void showWord(String w) {
    // Assumes words don't occur in multiple slots...
    text.setText(VerbLearn.context().lexicon().getWord(w).toString());
    currentWord = w;
  }



  public void saveLexicon() {
    if (VerbLearn.context().lexicon() != null) {
      String dirname = VerbLearn.DATA_BASE_DIR + "lang/" +
	VerbLearn.context().langslot() + "/lexicon/";
      File f = new File(dirname);
      if (!f.exists()) {
	f.mkdirs();
      }
      FileDialog fd = new FileDialog(VerbLearn.frame(), 
				     "Save Lexicon As...", 
				     FileDialog.SAVE);
      fd.setDirectory(dirname);
      if (!VerbLearn.context().lexicon().id().equals("New/Unnamed")) {
	fd.setFile(VerbLearn.context().lexicon().id());
      }
      fd.show();
      String fname = fd.getFile();
      if (fname != null) {
	VerbLearn.context().lexicon().setId(fname);
	String pathname = fd.getDirectory() + fname;
	try {
	  ObjectOutputStream os = new 
	    ObjectOutputStream(new FileOutputStream(pathname));
	  os.writeObject(VerbLearn.context().lexicon());
	  text.setText("Lexicon saved in file " + pathname + "\n");
	  VerbLearn.context().rereadLists();
	}
	catch (IOException e) {
	  text.setText("Could not save lexicon in " + pathname +
			 ": " + e.getMessage() + "\n");
	}
      }
    }
  }

  public void saveLexiconLog() {
    if ((VerbLearn.context().langslot() != null) &&
	(VerbLearn.context().lexicon() != null)) {
      String dirname = VerbLearn.DATA_BASE_DIR + "lang/" +
	VerbLearn.context().langslot() + "/output/";
      File f = new File(dirname);
      if (!f.exists()) {
	f.mkdirs();
      }
      FileDialog fd = new FileDialog(VerbLearn.frame(), 
				     "Save Text Version of Lexicon As...", 
				     FileDialog.SAVE);
      fd.setDirectory(dirname);
      fd.show();
      String fname = fd.getFile();
      if (fname != null) {
	String pathname = fd.getDirectory() + fname;
	try {
	  PrintWriter pw = new PrintWriter(new FileWriter(pathname));
	  pw.print("Language: " + VerbLearn.context().lang() +
		   "  Slots: " + VerbLearn.context().numSlots() + "\n\n" +
		   slotInfo.getText() + "\n");
 	  for (int si = 0; si < VerbLearn.context().numSlots(); si++) {
 	    pw.print("-------------------------------\n\n" +
		     "Slot " + si + " words:\n\n");
 	    String[] wl = VerbLearn.context().lexicon().slot(si).wordList();
 	    for (int wi = 0; wi < wl.length; wi++) {
 	      pw.print(VerbLearn.context().lexicon().slot(si)
		       .get(wl[wi]).toString() + "\n");
 	    }
 	  }
	  pw.close();
	  text.setText("Text version of lexicon saved in file " + 
		       pathname + "\n");
	}
	catch (IOException e) {
	  text.setText("Could not save text version of lexicon in " + 
		       pathname + ": " + e.getMessage() + "\n");
	}
      }
    }

  }
}
