package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;


///////////////////////////////////////////////////////////////////////
// Display current list of Scenarios for selection.  
// Optionally restrict the list to Scenarios with linking fstructs 
// (these may include "fakes") or to grounded Scenarios (no "fakes"),
// or to labelled Scenarios.

class ScenarioList extends List implements Observer {

  private static final int ROWS = 8;  // Number of rows to use.
  
  private boolean linkedOnly, groundedOnly, labelledOnly;

  // By default, allows multiple selections and is initially disabled.
  ScenarioList(boolean linkedOnly, boolean groundedOnly,
	       boolean labelledOnly) {
    super(ROWS, true);

    VerbLearn.context().addObserver(this);    
    setEnabled(false);
    VerbLearn.context().addObserver(this);
    setMasks(linkedOnly, groundedOnly, labelledOnly);
  }

  public void setMasks(boolean linkedOnly, boolean groundedOnly,
		  boolean labelledOnly) {
    this.linkedOnly = linkedOnly;
    this.groundedOnly = groundedOnly;
    this.labelledOnly = labelledOnly;
    update(VerbLearn.context().scenarios(), null);
  }

  public void update(Observable o, Object junk) {
    removeAll();
    Enumeration scnEnum = VerbLearn.context().scenarios().names();
    while (scnEnum.hasMoreElements()) {
      String scName = (String)scnEnum.nextElement();
      //      System.err.println("Sc seems to exist: " + scName);
      Scenario sc = VerbLearn.context().scenarios().get(scName);
      if (sc == null) {
	//	System.err.println("Not in scenario list: " + scName);
      }
      else {
      if ((!linkedOnly || sc.hasLinks()) &&
	  (!groundedOnly || sc.isGrounded()) &&
	  (!labelledOnly || sc.isLabelled())) {
	addItem(sc.id());
      }
      }
    }
  }
}
