package NTLSystem.verblearn;

import java.io.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import NTLSystem.support.*;


///////////////////////////////////////////////////////////////////////
// Training Panel

class TrainPanel extends Panel {

  private static final String INITIAL_TEXT = "Training log window\n\n";
  private Label title;
  private Panel buttonPanel1, buttonPanel2;
  private LangChoice langChoice;
  private ParamChoice paramChoice;
  private DatasetChoice datasetChoice;
  private LexiconChoice lexiconChoice;
  private Button paramEdit, datasetEdit, doIt;
  private ScrollingTextArea log;
  private Button clear, save;

  private ParamsPanel paramsPanel; // nc
  //   private Vector paramsValues; // nc

  public TextArea log() { return log; }

  TrainPanel() {
    setLayout(new GridBagLayout());
    title = new Label("Training Window");
    title.setFont(new Font("Helvetica", Font.BOLD, 18));
    Constrain.add(this, title, 0, 0, 3, 1, 10, 10, 10, 10);
    buttonPanel1 = new Panel();
    buttonPanel1.setLayout(new GridBagLayout());
    langChoice = new LangChoice();
    Constrain.add(buttonPanel1, langChoice, 0, 0, 1, 1, 0, 0, 0, 10);
    paramChoice = new ParamChoice();
    Constrain.add(buttonPanel1, paramChoice, 1, 0, 1, 1, 
		  GridBagConstraints.NONE, GridBagConstraints.EAST, 
		  0, 10, 0, 10);
    paramEdit = new Button("View/Edit");
    Constrain.add(buttonPanel1, paramEdit, 2, 0, 1, 1, 
		  GridBagConstraints.NONE, GridBagConstraints.WEST, 
		  0, 10, 0, 0);
    Constrain.add(this, buttonPanel1, 0, 1, 3, 1, 10, 5, 10, 5);
    buttonPanel2 = new Panel();
    buttonPanel2.setLayout(new GridBagLayout());
    lexiconChoice = new LexiconChoice();
    Constrain.add(buttonPanel2, lexiconChoice, 0, 0, 1, 1, 0, 0, 0, 10);
    datasetChoice = new DatasetChoice();
    Constrain.add(buttonPanel2, datasetChoice, 1, 0, 1, 1, 
		  GridBagConstraints.NONE, GridBagConstraints.EAST, 
		  0, 10, 0, 10);
    datasetEdit = new Button("View/Edit");
    Constrain.add(buttonPanel2, datasetEdit, 2, 0, 1, 1, 
		  GridBagConstraints.NONE, GridBagConstraints.WEST, 
		  0, 10, 0, 0);
    Constrain.add(this, buttonPanel2, 0, 2, 3, 1, 10, 5, 10, 5);
    doIt = new Button("Do It !");
    doIt.setFont(new Font("Helvetica", Font.PLAIN, 18));
    Constrain.add(this, doIt, 0, 3, 3, 1, 10, 10, 10, 10);
    log = new ScrollingTextArea(INITIAL_TEXT, 30, 80);
    Constrain.add(this, log, 0, 4, 4, 1, GridBagConstraints.BOTH,
		  GridBagConstraints.CENTER, 5, 10, 5, 10);
    clear = new Button("Clear Log");
    Constrain.add(this, clear, 0, 5, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.WEST, 5, 10, 10, 10);
    save = new Button("Save Log...");
    Constrain.add(this, save, 2, 5, 1, 1, GridBagConstraints.NONE,
		  GridBagConstraints.EAST, 5, 10, 10, 10);
  }

  public boolean action(Event e, Object junk) {
    if (e.target == doIt) {
      train();
      return true;
    }
    else if (e.target == paramEdit) {
      editParams();
      return true;
    }
    else if (e.target == datasetEdit) {
      editDataset();
      return true;
    }
    else if (e.target == save) {
      if (VerbLearn.isApplication()) {
	saveLog();
      }
      else {
	log.append("Sorry, applet version can't save files.\n");
      }
      return true;
    }
    else if (e.target == clear) {
      log.setText(INITIAL_TEXT);
      return true;
    }

    // nc added many more buttons -- use inner class instead?
    return super.action(e, junk);
  }

  private void train() {
    if (VerbLearn.context().params() == null) {
      log.append("You must choose a parameter file before training.\n");
      return;
    }
    if (VerbLearn.context().dataset() == null) {
      log.append("You must choose a dataset before training.\n");
      return;
    }
    if (VerbLearn.context().lexicon() == null) {
      log.append("You must choose an initial lexicon before training.\n");
      return;
    }
    log.append(VerbLearn.context().params().toString());
    log.append("\nBeginning training...\n");
    for (int i = 0; i < VerbLearn.context().params().trainingPasses(); i++) {
      Enumeration scEnum = VerbLearn.context().dataset().trainEnum();
      while (scEnum.hasMoreElements()) {
	String scName = (String)scEnum.nextElement();
	Scenario sc = VerbLearn.context().scenarios().get(scName);
			
	// nc add to prevent crash!
	if (VerbLearn.context().scenarios().containsKey(scName)) {
	  if (sc.isLabelled()) {
	    log.append("Incorporating scenario " + sc.id() + ":\n");
	    // FIXME_jsled: This should really be the unified Fstruct
	    // instead of finalLink() [which doesn't exist].
	    VerbLearn.context().lexicon().incorporate(sc.finalLink(), 
						    sc.label());
	  // SOME CODE TO HELP DIAGNOSE LINUX MEMORY LEAK PROBLEMS:
	  // System.gc();
	  // long mTotal = Runtime.getRuntime().totalMemory();
	  // long mFree = Runtime.getRuntime().freeMemory();
	  // System.err.println("total=" + mTotal + ", free=" + mFree +
	  //		        ", thus used=" + (mTotal - mFree));
	  }
	  else {
	    log.append("WARNING: scenario " + sc.id() + " is unlabelled.\n");
	  }
	}
	else {
	  log.append("WARNING: non-existing scenario" + scName +
		     " not in scenario set.\n");
	}
      }
    }
    VerbLearn.context().lexicon().synch();
    log.append("Done training.\n");
    // Tell the LexiconPanel and ObeyPanel that the lexicon has changed:
    VerbLearn.lexiconPanel().rebuildLexiconGrid();
    VerbLearn.obeyPanel().rebuildCommandEntryGrid();
    getToolkit().beep();
  }

  private void editParams() {
    if (VerbLearn.context().params() == null) {
      log.append("Can't view/edit parameter set until one's been chosen!\n");
      return;
    }

    /*    TextEditDialog fed = new TextEditDialog(
                                 VerbLearn.frame(), "Edit Parameters",
			         VerbLearn.context().params().toString(),
				 true);
    */
    // NC replace above with new params panel

    // creates paramsPanel, which keeps hashtable paramsTable of new OBJECTS value
    paramsPanel = new ParamsPanel(VerbLearn.context().params());
    EditDialog fed = new EditDialog(VerbLearn.frame(),
				    "Edit Parameters", paramsPanel, true);
    fed.show();

    if (fed.apply()) {
      /* nc rm     
	VerbLearn.context().setParams(new Params(
        new NestedStreamTokenizer(new StringReader(fed.getText())),
	VerbLearn.context().params().id()));
	*/

      // incorporate _changes_ from the paramsTable(); value field = Object
      paramsPanel.getChanges();
      VerbLearn.context().setParams(paramsPanel.paramsTable());
      log.append("Changes to parameters noted.\n");

      if (fed.save()) {
	if (VerbLearn.isApplication()) {
	  if (VerbLearn.context().params().isModified()) {
	    if (VerbLearn.context().params().save()) {
	      log.append("Saved parameters.\n");
	    }
	    else {
	      log.append("Failed to save parameters.\n");
	    }
	  }
	}
	else {
	  log.append("Sorry, applet version can't save files.\n");
	}
      }
      else {
	log.append("Viewed parameters but made no changes.\n");
      }
    }




      }
// 	if (VerbLearn.isApplication()) {
// 	  FileDialog fd = new FileDialog(VerbLearn.frame(),
// 					 "Save New Parameters As...",
// 					 FileDialog.SAVE);
// 	  String dirname = VerbLearn.DATA_BASE_DIR + "param/";
// 	  fd.setDirectory(dirname);
// 	  fd.setFile(VerbLearn.context().params().id());
// 	  fd.show();
// 	  String fname = fd.getFile();
// 	  if (fname != null) {
// 	    VerbLearn.context().params().setId(fname);
// 	    String pathname = dirname + fname;
// 	    try {
// 	      PrintWriter pw = new PrintWriter(new FileWriter(pathname));
// 	      pw.print(VerbLearn.context().params().toString());
// 	      pw.close();
// 	      VerbLearn.context().rereadLists();
// 	      log.append("Saved parameters in " + pathname + ".\n");
// 	    }
// 	    catch (IOException e) {
// 	      log.append("Failed to save parameters in " + pathname + ".\n");
// 	    }
//	  }
//    }
//	else {
//	  log.append("Sorry, applet version can't save files.\n");
    //    }
    //    else {
    //      log.append("Viewed parameters but made no changes.\n");
    //    }
    //  }
  
  private void editDataset() {
    if (VerbLearn.context().dataset() == null) {
      log.append("Can't view/edit dataset until one's been chosen!\n");
      return;
    }
    TextEditDialog fed = new TextEditDialog(
                                 VerbLearn.frame(), "Edit Dataset",
			         VerbLearn.context().dataset().toString(),
				 true); 
    fed.show();
    if (fed.apply()) {
      VerbLearn.context().setDataset(new Dataset(
        new NestedStreamTokenizer(new StringReader(fed.getText())),
	VerbLearn.context().dataset().id()));
      log.append("Changes to dataset noted.\n");
      if (fed.save()) {
	if (VerbLearn.isApplication()) {
	  FileDialog fd = new FileDialog(VerbLearn.frame(),
					 "Save New Dataset As...",
					 FileDialog.SAVE);
	  String dirname = VerbLearn.DATA_BASE_DIR + "dataset/";
	  fd.setDirectory(dirname);
	  fd.setFile(VerbLearn.context().dataset().id());
	  fd.show();
	  String fname = fd.getFile();
	  if (fname != null) {
	    VerbLearn.context().dataset().setId(fname);
	    String pathname = dirname + fname;
	    try {
	      PrintWriter pw = new PrintWriter(new FileWriter(pathname));
	      pw.print(VerbLearn.context().dataset().toString());
	      pw.close();
	      VerbLearn.context().rereadLists();
	      log.append("Saved dataset in " + pathname + ".\n");
	    }
	    catch (IOException e) {
	      log.append("Failed to save dataset in " + pathname + ".\n");
	    }
	  }
	}
	else {
	  log.append("Sorry, applet version can't save files.\n");
	}
      }
    }
    else {
      log.append("Viewed dataset but made no changes.\n");
    }
  }

  private void saveLog() {
    if (VerbLearn.context().langslot() != null) {
      String dirname = VerbLearn.DATA_BASE_DIR + "lang/" +
	VerbLearn.context().langslot() + "/output/";
      File f = new File(dirname);
      if (!f.exists()) {
	f.mkdirs();
      }
      FileDialog fd = new FileDialog(VerbLearn.frame(), 
				     "Save Training Log As...", 
				     FileDialog.SAVE);
      fd.setDirectory(dirname);
      fd.show();
      String fname = fd.getFile();
      if (fname != null) {
	String pathname = fd.getDirectory() + fname;
	try {
	  PrintWriter pw = new PrintWriter(new FileWriter(pathname));
	  pw.print(log.getText());
	  pw.close();
	  log.append("This log saved in file " + pathname + "\n");
	}
	catch (IOException e) {
	  log.append("Could not save this log in " + pathname + ": " + 
		     e.getMessage() + "\n");
	}
      }
    }
  }
}

